package net.sf.sidaof.dao;

import java.util.Collection;
import java.util.List;

/**
 * Base interface for all DAOs. To use, extend it in DAO interfaces.
 * 
 * @param <K>
 *            The primary key class of the entity, e.g. Long.
 * @param <E>
 *            The entity class the DAO persists, e.g. Person.
 */
public interface Dao<K, E> {
    /**
     * Save the entity.
     * 
     * @param entity
     *            The persistence entity to save.
     */
    void persist(E entity);

    /**
     * Save the Collection of entities.
     * 
     * @param entities
     *            The Collection of persistence entities to save.
     */
    void persist(Collection<E> entities);

    /**
     * Merge the entity.
     * 
     * @param entity
     *            The persistence entity to merge.
     * @return The merged entity.
     */
    E merge(E entity);

    /**
     * Delete the entity.
     * 
     * @param entity
     *            The persistence entity to delete.
     */
    void remove(E entity);

    /**
     * Find an entity by its id, and return null if not found.
     * 
     * @param id
     *            The id (primary key) of the entity to find.
     * @return The entity for the specified id, null when entity not found.
     */
    E findById(K id);

    /**
     * Find all entities.
     * 
     * @return List of all entities.
     */
    List<E> findAll();

    /**
     * Find all entities starting in the resultset specified by startPosition
     * and obtaining up to the number specified by maxResult. This method is
     * useful for pagination of results.
     * 
     * @param startPosition
     *            The starting position in the result set to retrieve.
     * @param maxResult
     *            The maximum number of rows to retrieve.
     * @return List of a subset of all entities, starting with startPosition and
     *         containing up to maxResult entries.
     */
    List<E> findAll(int startPosition, int maxResult);

    /**
     * Synchronize the underlying persistent store with persistable state held
     * in memory.
     */
    void flush();

    /**
     * Clear the in-memory persistable state, including unflushed changes.
     */
    void clear();

    /**
     * Check if the entity belongs to the current persistence context.
     * 
     * @param entity
     *            The persistence entity to see if exists.
     * @return true if the entity belongs to the current persistence context.
     */
    boolean contains(E entity);

    /**
     * Reload the entity.
     * 
     * @param entity
     *            The persistence entity to reload from the persistence context.
     */
    void refresh(E entity);
}
